/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2008 The LS-Universe Team

    Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "Utility.h"

#include <shlwapi.h>

#include "Label.h"

extern POINT ptMinScreen;

extern ProcessStatsRequestFunc m_pProcessStatsRequest;

HINSTANCE xLSExecute(HWND Owner, LPCSTR szCommand, int nShowCmd)
{
	CString process = szCommand;
	process.MakeLower();
	if (process.Find( "http:" ) != -1 && process.Find( "!" ) == -1)
	{
		SHELLEXECUTEINFO ShExecInfo = {0};
		ShExecInfo.cbSize = sizeof(SHELLEXECUTEINFO);
		ShExecInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
		ShExecInfo.hwnd = NULL;
		ShExecInfo.lpVerb = "";
		ShExecInfo.lpFile = szCommand;		
		ShExecInfo.lpParameters = "";	
		ShExecInfo.lpDirectory = NULL;
		ShExecInfo.nShow = nShowCmd;
		ShExecInfo.hInstApp = NULL;	
		ShellExecuteEx(&ShExecInfo);

		return ShExecInfo.hInstApp;
	}
	else
	{
		return LSExecute(Owner, szCommand, nShowCmd);
	}
}

// trim leading and trailing whitespace
string trim(const string &toTrim, bool bRemoveQuotes)
{
	int start = 0;
	int end = toTrim.length() - 1;
	
	while(start <= end && isspace(toTrim[start]))
		start++;

	if ( bRemoveQuotes && start <= end && toTrim[start] == '\"')
		start++;
	
	while(start <= end && isspace(toTrim[end]))
		end--;

	if ( bRemoveQuotes && start <= end && toTrim[end] == '\"')
		end--;
	
	return toTrim.substr(start, end - start + 1);
}

string IntToString(int iInput)
{
	char buffer[32] = {0};
	itoa( iInput, buffer, 10 );
	return (string) buffer;
}

int SplitString(const string &process, string &one, string &two, string &three, string &four, string &five, string &six, string &seven)
{
	int count = 0;
	one = "";
	two = "";
	three = "";
	four = "";
	five = "";
	six = "";
	seven = "";
	char cone[256] = {0};
	char ctwo[256] = {0};
	char cthree[256] = {0};
	char cfour[256] = {0};
	char cfive[256] = {0};
	char csix[256] = {0};
	char cseven[256] = {0};
	char *cbuffers[] = {cone, ctwo, cthree, cfour, cfive, csix, cseven};
		
	int numTokens = LCTokenize( process.c_str(), cbuffers, 7, NULL);
	if( numTokens >= 1 )
	{
		one = cone;
		count++;
		
		if ( numTokens >= 2 )
		{
			two = ctwo;
			count++;
		
			if ( numTokens >= 3 )
			{
				three = cthree;
				count++;
			
				if ( numTokens >= 4 )
				{
					four = cfour;
					count++;

					if ( numTokens >= 5 )
					{
						five = cfive;
						count++;

						if ( numTokens >= 6 )
						{
							six = csix;
							count++;
		
							if ( numTokens >= 7 )
							{
								seven = cseven;
								count++;
							}
						}
					}
				}
			}
		}
	}

	return count;
}

string iGetxRCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	char szValue[MAX_LINE_LENGTH] = { 0 };
	GetxRCString(szValue, uniqueKey.c_str(), baseName.c_str(), defaultVal.c_str());
	return (string) szValue;
}

string iGetx2RCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	char szValue[MAX_LINE_LENGTH] = { 0 };
	Getx2RCString(szValue, uniqueKey.c_str(), baseName.c_str(), defaultVal.c_str());
	return (string) szValue;
}

string iGetxRCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	char szValue[MAX_LINE_LENGTH] = { 0 };
	GetxRCLine(szValue, uniqueKey.c_str(), baseName.c_str(), defaultVal.c_str());
	return (string) szValue;
}

string iGetx2RCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	char szValue[MAX_LINE_LENGTH] = { 0 };
	Getx2RCLine(szValue, uniqueKey.c_str(), baseName.c_str(), defaultVal.c_str());
	return (string) szValue;
}

void PrefixedLSSetVariable(string prefix, string var, string value)
{
	if ( !var.empty() )
		prefix.append(var);

	LSSetVariable(prefix.c_str(), value.c_str());
}

// retrieve a list of names from step.rc
StringList GetxRCNameList(const string &uniqueKey)
{
	StringList resultList;
	LPVOID f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];
		char temp[32], names[MAX_LINE_LENGTH];
		char *buffers[] = {temp};

		while(LCReadNextConfig(f, uniqueKey.c_str(), lineBuffer, MAX_LINE_LENGTH))
		{
			if (LCTokenize(lineBuffer, buffers, 1, names) >= 1)
			{
				char szBuffer[MAX_LINE_LENGTH] = {0};
				LPCSTR pszNextToken = names;

				while ( GetToken(pszNextToken, szBuffer, &pszNextToken, false) )
					resultList.insert(resultList.end(), szBuffer);
			}
		}

		LCClose(f);
	}

	return resultList;
}

StringList GetxRCLineList(const string &uniqueKey)
{
	StringList resultList;
	LPVOID f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];
		char temp[64], configline[MAX_LINE_LENGTH];
		char *buffers[] = {temp};

		while(LCReadNextConfig(f, uniqueKey.c_str(), lineBuffer, MAX_LINE_LENGTH))
		{
			if (LCTokenize(lineBuffer, buffers, 1, configline) >= 1)
				resultList.insert(resultList.end(), (string) configline);
		}
		
		LCClose(f);
	}

	return resultList;
}

vector<string> split(const string &str, const string &sep)
{
	vector<string> substrs;

	string check = "";
	string keeper = "";
	int start = 0;
	int end = str.find(sep, start);
	
	while(end != string::npos)
	{
		if (sep == ";")
		{
			if ( end >= 3 )
				check = str.substr(end-3, 3);
			else
				check = "";

			if (_stricmp(check.c_str(), "uot") == NULL || _stricmp(check.c_str(), "&lt") == NULL || _stricmp(check.c_str(), "&gt") == NULL || _stricmp(check.c_str(), "amp") == NULL)
			{
				keeper.append(str.substr(start, end - start - sep.length() + 2));
			}
			else if (!keeper.empty())
			{
				keeper.append(str.substr(start, end - start - sep.length() + 1));
				substrs.push_back(keeper);
				keeper = "";
			}
			else
			{
				substrs.push_back(str.substr(start, end - start - sep.length() + 1));
			}
		}
		else
		{
			substrs.push_back(str.substr(start, end - start));
		}
		
		start = end + sep.length();
		end = str.find(sep, start);
	}

	if (!keeper.empty())
	{
		keeper.append(str.substr(start));
		substrs.push_back(keeper);
	}
	else
	{
		substrs.push_back(str.substr(start));
	}

	return substrs;
}

BOOL GetClipboardText(HWND hwndOwner, LPTSTR pszBuffer, UINT cchBuffer)
{
	HGLOBAL hMem;
	LPVOID pvMem;
	BOOL fResult = FALSE;

	if(IsClipboardFormatAvailable(CF_TEXT))
	{
		if(OpenClipboard(hwndOwner))
		{
			hMem = GetClipboardData(CF_TEXT);

			if(hMem)
			{
				pvMem = GlobalLock(hMem);
				lstrcpyn(pszBuffer, (LPCTSTR) pvMem, cchBuffer);
				GlobalUnlock(hMem);
				fResult = TRUE;
			}

			CloseClipboard();
		}
	}

	return fResult;
}

BOOL SetClipboardText(HWND hwndOwner, LPCTSTR pszText)
{
	UINT cbData;
	HGLOBAL hMem;
	LPVOID pvMem;
	BOOL fResult = FALSE;

	if(OpenClipboard(hwndOwner))
	{
		if(EmptyClipboard())
		{
			cbData = (lstrlen(pszText) + 1) * sizeof(TCHAR);
			hMem = GlobalAlloc(GMEM_MOVEABLE, cbData);

			if(hMem)
			{
				pvMem = GlobalLock(hMem);
				memcpy(pvMem, pszText, cbData);
				GlobalUnlock(hMem);
				fResult = SetClipboardData(CF_TEXT, hMem) ? TRUE : FALSE;

			}
		}

		CloseClipboard();
	}

	return fResult;
}

string processLabelText(const string &text, Label *label, bool *dynamic)
{
	string strNormalProcessing = text;

	if ( label != NULL && !text.empty() )
	{
		string strLowerProcessing = lowerCase(text);
		
		int iPos = strLowerProcessing.find("[labelname]");

		while ( iPos != -1 )
		{
			strNormalProcessing.replace(iPos, 11, label->getName());
			strLowerProcessing.replace(iPos, 11, label->getName());
			iPos = strLowerProcessing.find("[labelname]");
		}

		iPos = strLowerProcessing.find("[labeltext]");

		while ( iPos != -1 )
		{
			strNormalProcessing.replace(iPos, 11, label->getText());
			strLowerProcessing.replace(iPos, 11, label->getText());
			iPos = strLowerProcessing.find("[labeltext]");
		}
	}

	if ( m_pProcessStatsRequest && !text.empty() )
	{
		char tmp[4096] = {0};
		if ( m_pProcessStatsRequest(tmp, strNormalProcessing.c_str()) )
		{
			if (dynamic) *dynamic = true;
		}
		else
		{
			if (dynamic) *dynamic = false;
		}

		return (string) tmp;
	}
	else
	{
		return text;
	}
}

string lowerCase(const string &s)
{
	string result = "";
	
	int length = s.length();
	int i = 0;
	
	while(i < length)
		result.append(1, tolower(s[i++]));
	
	return result;
}
